//----------------------------------------------------------------------------

//DSS-90
//   DS18B20

//   DS18B20   1-
//,   .
//     OWP.
//      ,
//     ,
//    4.7 .
//    
//   (     ),
//      .
//    :
//#define OWP (1 << Pxn)
//#define Port_OWP_OUT (DDRx |= OWP)
//#define Port_OWP_IN (DDRx &= ~OWP)
//#define Pin_OWP   (PINx & OWP)
// DS18B20     12- 
//( R0, R1     ,
//  ).

//    0.1C.
//    2 , CRC  .

//        
//  DS18B20_Init().  DS18B20_Exe() 
//   ,    ,
//        
//    Temp.

//       ( 
//     0.5 ),  
//        DS18B20_Exe()
//.      .  
//      ,  
//         .

//          
//DS18B20_GetT().     16-  
//         .
// ,     , 
// T_NORDY.      
//  T_ERROR.

//----------------------------------------------------------------------------

#include "Main.h"
#include "DS18B20.h"

//------------------------------ : ----------------------------------

#define TCONV 800      //  , 

//----------------------------- : ----------------------------------

static int Temp;       // 
static char TState;    //  

//--------------------------  : ------------------------------

bool TReset(void);     //  
char TByte(char dat);  /// 
bool TBit(bool b);     /// 

//---------------------  : ----------------------------

void DS18B20_Init(void)
{
  Temp = T_NORDY;      //   
  TState = 0;          // 
}

//-----------------------  : -----------------------------

void DS18B20_Exe(bool t)
{
  static int TTimer = 0;              // 
  static int T;                       // 
  if(t)
  {
    if(TTimer)                        //    ,
    {
      TTimer--;                       // (  )
    }
    else                              //  
    {
      switch(TState)
      {
      case 0: if(TReset())            // RESET
                TState++;             //  ,  
                  else TState = 0xFF; // ,   
              break;
      case 1: TByte(0xCC);            // SKIP ROM
              TState++;               // 
              break;               
      case 2: TByte(0x44);            // START CONVERSION
              TTimer = ms2sys(TCONV); //  
              TState++;               // 
              break;
      case 3: if(TReset())            // RESET
                TState++;             //  ,  
                  else TState = 0xFF; // ,   
              break;
      case 4: TByte(0xCC);            // SKIP ROM
              TState++;               // 
              break;
      case 5: TByte(0xBE);            // READ SCRATCHPAD
              TState++;               // 
              break;
      case 6: T = TByte(0xFF);        // TL
              TState++;               // 
              break;
      case 7: T |= TByte(0xFF) << 8;  // TH
              TState++;               // 
              break;
      case 8: Temp = 10 * T / 16;     // 
              TState = 0;             //   
              break;
      default: TTimer = ms2sys(TCONV); // 
              Temp = T_ERROR;         // 
              TState = 0;             //   
      }
    }
  }
}

//---------------------   : ---------------------------

bool TReset(void)
{
  char si;
  Port_OWP_0;
  Port_OWP_OUT;                //OWP <- 0
  Delay_us(500);               //delay 500 uS
  si = __save_interrupt();
  __disable_interrupt();       // 
  Port_OWP_IN;                 //OWP <- 1
  Delay_us(14);                //delay 14 uS
  if(Pin_OWP)                  // OWP = 0,  
  {
    Delay_us(52);              //delay 52 uS
    if(!Pin_OWP)               // OWP = 1,  
    {
      __restore_interrupt(si); // 
      Delay_us(250);           //delay 250 uS
      if(Pin_OWP)              // OWP = 0,  
      {
        return(1);             //  ,  
      }
    }
  }
  __restore_interrupt(si);     //    
  return(0);
}

//-------------- /    : ------------------

char TByte(char dat)
{
  char res;
  for(char i = 0; i < 8; i++)
  {
    res = res >> 1;
    if(TBit(dat & 1)) res |= 0x80;
     else res &= ~0x80;
    dat = dat >> 1;
  }
  return(res);
}

//--------------- /    : ------------------

bool TBit(bool b)
{
  char si;
  si = __save_interrupt();
  __disable_interrupt();       // 
  Port_OWP_0;
  Port_OWP_OUT;                //OWP <- 0
  Delay_us(2);                 //delay 2 uS
  if(b) Port_OWP_IN;           //bit = 1, OWP <- 1
  Delay_us(13);                //delay 13 uS
  bool owp = Pin_OWP;          // 
  Delay_us(45);                //delay 45 uS
  Port_OWP_1;                  //force pullup
  Port_OWP_OUT;
  __restore_interrupt(si);     // 
  Delay_us(2);                 //delay 2 uS
  return(owp);
}

//-------------------------  : ------------------------------

int DS18B20_GetT(void)
{
  return(Temp);
}

//----------------------------------------------------------------------------
